/*
____________________________________________________________________________


rash is Copyright 1992 by Predrag S. Bundalo
See Legal Notice below for licensing restrictions.

Title      : rash -- restricted shell
Version    : 1.0.1
Author     : Predrag S. Bundalo (pred@iitmax.iit.edu)
Date       : August 17th, 1992
Environment: COHERENT v4.0.1 K&R C
Description: rash is a restricted user shell.  It allows commands to be
History    : September 12th, 1892 Udo Munk
	     Bug fixed for compiling with -DCD under Coherent

	executed from one directory only--/rbin by compiled default.  It
	allows no redirection (input or output), piping, or job control.

	The ideal environment for such a program is for restricting expired
	user accounts by only allowing them access to commands that will help
	users transfer important files from their accounts to someplace else
	before the account is removed.

	Note that access to MAIL is not recommended.

	rash is currently in use on an Encore Multimax supermini parallel
	computer at the Academic Computing Center at the Illinois Institute
	of Technology in Chicago.

	The standard less-secure (but more featureful) restricted bourne shell
	is also used this way at the National Center for Supercomputing
	Applications at the University of Illinois in Urbana-Champaign.

_______________

Compiling rash:
_______________

For CRAY Supercomputers running UNICOS (6.1): (Y-MP in particular)

% cc -O -o rash [-DCD] [-DVERBOSE] -DNOPATHPROC rash.c -h tolerant

For NeXT 68040 systems running NeXTSTEP 2.1+:

% cc -O -o rash [-DCD] [-DVERBOSE] -DNOPATHPROC rash.c

For systems with a path() system call (such as COHERENT v4.0.x):
(Note that COHERENT users have their choice of using either path() below,
 or access() further below.)

% cc rash.c -o rash

Or to compile a verbose version that includes credits, compile with:

% cc -DVERBOSE -o rash

Or to compile with verbose descriptions, and enable the "cd" and "pwd"
built-in's:

% cc -DVERBOSE -DCD -o rash

------

And for systems without path() but with access() simply add the
NOPATHPROC define to the compile line:

% cc -DNOPATHPROC [-DVERBOSE] [-DCD] -O -o rash

For example:

% cc -DNOPATHPROC -DVERBOSE -DCD -o rash

will compile rash on a system with no path() function with verbose
explanations, and the "cd" & "pwd" built-in's enabled.

Note: the brackets "[]" mean the enclosed argument is optional.
Also note, you may want to "strip rash" to trim its size after compiling.

rash should be installed with owner (and group where applicable) "bin"
and mode 711.

The /rbin directory should have owner (and group) "bin" and mode 755.

Also, you should create a file (MESSAGE_FILE) in the /rbin directory that
describes the current situation for users.  A sample is included.
______________________________________________

Typical /rbin file-contents: (These are links)
______________________________________________

lr-x--x--x  2 bin        bin           5620 Mon Jan 22  1990 cat
lr-x--x--x  2 bin        bin         289728 Tue Jul 14 10:55 ckermit
lr-x--x--x  4 bin        bin          13548 Thu Jun 11 02:33 compress
lrwx--x--x  3 bin        bin          10064 Tue Aug 18 02:13 f
lrwx--x--x  3 bin        bin          10064 Tue Aug 18 02:13 finger
lrwx--x--x  2 bin        bin         246452 Fri Dec 25  1992 ftp
lr-x--x--x  2 bin        bin          12050 Tue Dec 17  1991 kermit
lr-x--x--x  6 bin        bin          18772 Thu Jul 16 10:11 ls
lr-s--s--x  2 root       root          9056 Thu Jun 11 02:35 rm
lrwx--x--x  2 bin        bin          14806 Thu Aug 19 02:35 sz
lrwx--x--x  2 bin        bin          61676 Fri Aug 14 01:29 tar
lr-x--x--x  2 bin        bin          12716 Fri May 31  1991 uudecode
lr-x--x--x  2 bin        bin           5878 Fri May 31  1991 uuencode
lr-s--s--x  2 root       root          7386 Thu Apr  5  1990 write

And specially-written restricted programs, such as "red", a restricted
editor.

________________________________

Files NOT to be placed in /rbin:
________________________________

o	vi, ex, ed
o	more, less
o	mail, elm, mailx
o	sh, csh, tcsh, ksh, bash, ash, zsh (or any other shell)
o	Any file that allows the user to start a shell.

_____________

Legal Notice:
_____________

rash is Copyright 1992 by Predrag S. Bundalo

This program may be freely distributed so long as this copyright notice
is intact and the source code is included.  Commercial use is 
prohibited without expressed written consent from the author.

Locally-modified versions of rash must have changes clearly marked before
they may be redistributed.

The author requests a copy of all local modifications to rash before
redistribution in order to possibly incorporate them in new versions.
This will facilitate "official" releases of the software.

_________________

Legal Disclaimer:
_________________

rash is given "as is".  There are no warranties or guarantees.  The author
is not responsible for any problems that occur either through direct or indirect
use of rash.

______________________________________________________________________________

Sample rash.msg file: (to be placed in your /rbin directory with read permis.)
______________________________________________________________________________
:
::
::: This account has been disabled pending deletion.  If you don't think
::: this account should be deleted, please contact the Academic Computing
::: Center as soon as possible (at x5962).
:::
::: iitmax accounts are only available to students for research or classwork-
::: related activities, as approved by an instructor or advisor.
:::
::: Once this account is actually deleted, there may be a fee of up to $50
::: to have it restored.  Not deleting it in the first place, however,
::: is free.
:::
::: You have until then to move your important files to another computer.
::
:

_________________

Trademark notice:
_________________

UNIX is a registered trademark of UNIX Software Laboratories, an AT&T
company.
____________________________________________________________________________
*/

#include <stdio.h>
#ifdef NOPATHPROC
#if COHERENT
#include <access.h>
#else
#ifdef CRAY
#include <sys/unistd.h>
#else
#include <sys/file.h>
#endif CRAY
#endif COHERENT
#else
#include <path.h>
#endif NOPATHPROC

#include <string.h>
#include <ctype.h>
#include <signal.h>
#ifdef CD
#define MAXPATHLEN	128	/* A default */
#ifdef COHERENT
#undef MAXPATHLEN
#include <path.h>
#define MAXPATHLEN	MAXPATH	/* use Coherent's MAXPATHLEN (Udo Munk) */
#else
#include <sys/param.h>
#endif COHERENT
#endif CD

#define MAX_ARGS	256
/* This is the original path for rash */
/*
#define BIN_PATH	"/rbin/"
#define MESSAGE_FILE	"/rbin/rash.msg"
*/
/* I want to have it in my user filesystem (Udo Munk) */
#define BIN_PATH	"/usr/rbin/"
#define MESSAGE_FILE	"/usr/rbin/rash.msg"
#define VERSION		"1.0.1"
#define DATE		"August 10th, 1992"
#define EMAILTO		"pred@iitmax.iit.edu"

#ifndef lint
char copyright[] =
"@(#)Copyright (c) 1992 Predrag S. Bundalo.\n\
See included source for distribution restrictions.\n";

static char sccsid[] =
"@(#)rash.c	1.0.1 (Predrag S. Bundalo) 8/21/92";
#endif ndef lint

/*
**	Functions follow.
*/

signal_handler(){
	fprintf(stderr,"\n\007::: Exiting rash.\n" );
	exit( 2 );
}

main() {
FILE *ifp;	/* Message file pointer. */
/* char *prompt = "[33;40m$ "; */ /* Amber(brown) on black. */
char *prompt = "$ ";	/* No color. */
char line[1024];
char *p;	/* Pointer */
char token[1024];
char *args[MAX_ARGS];	/* Argument list. */
char aargs[MAX_ARGS][80];
int numargs = 0;
int pid;
#ifdef CD
char home[MAXPATHLEN];
char cwd [MAXPATHLEN];
#endif CD

/*
**	Set up an interrupt signal handler.
*/
	if( signal( SIGINT, signal_handler ) < 0 ){
		perror("signal()");
		exit( -1 );
	}

fprintf(stderr,":\n::\n::: rash (restricted shell) v%s\n",VERSION);

#ifdef VERBOSE
fprintf(stderr,"-- Press <return> to continue --");
gets(line);
fprintf(stderr,":::\n::: Copyright 1992 by Predrag S. Bundalo\n" );
fprintf(stderr,"::: %s (email to: %s)\n:::\n",DATE,EMAILTO );
fprintf(stderr,"::: Note: commands may only be executed from the\n");
fprintf(stderr,":::       %s directory.\n:::\n",BIN_PATH);
fprintf(stderr,":::       Input and output redirection, pipes, and\n");
fprintf(stderr,":::       job control are disabled.\n:::\n");
fprintf(stderr,":::       Type \"ls %s\" for available commands.\n",BIN_PATH);
#endif defined VERBOSE
fprintf(stderr,"::\n:\n");

#ifdef CD
/* Get the user's home directory--save it. */
#if COHERENT
if( getcwd(home,MAXPATHLEN) == NULL ){
#else
if( getwd(home) == NULL ){
#endif COHERENT
wderr:	fprintf(stderr,"rash: get[c]wd() : %s.\n",home);
	exit(-2);
}
strcpy(cwd,home);
#endif CD

fprintf(stderr,"-- Press <return> to continue --");
gets(line);

/* Get and print a message file. */
if( !(ifp=fopen(MESSAGE_FILE,"r")) )
	perror(MESSAGE_FILE);
else {
	while( fgets(line,1024,ifp) != NULL )
		printf("%s",line);
	fclose(ifp);
}

/* Begin an infinite loop to get and execute allowed user commands. */
while ( 1 ) {
	numargs = 0;
	/* Display the prompt. */
	printf("%s",prompt);

	/* Get a line of input. */
	if( fgets( line, 1024, stdin ) == NULL )
		signal_handler();

/* Break it up into tokens. */
	p = line;

	/* Skip whitespaces. */
	while( *p == ' ' || *p == '\t' )
		p++;

	if( *p == '\n' ) /* End of line, so no command was entered. */
		continue;

	/* Get the first argument--the command. */
	if( sscanf(p,"%s",token) != 1 ){
err:		fprintf(stderr,"rash: command expected.\n");
		exit(1);
	}

	/* Check for a legal path. */

	if( !isalpha(token[0]) ){
		fprintf(stderr,"%c : invalid command symbol.\n",token[0]);
		continue;
	}

#ifdef CD
/*
**	Check to see if the user wants to use the built-in's "cd"
**	or "pwd".
*/
	if( !strcmp("cd",token) ){
		p += strlen(token)+1;
		while( isspace(*p) )
			p++;
		if( *p == '\0' ){
			if( chdir(home) )
				perror("rash ");
#if COHERENT
			if( getcwd(cwd,MAXPATHLEN) == NULL )
#else
			if( getwd(cwd) == NULL )
#endif COHERENT
				goto wderr;
			continue;
		}
		*(p+strlen(p)-1) = '\0';
		if( chdir(p) )
			perror("rash ");
#if COHERENT
		if( getcwd(cwd,MAXPATHLEN) == NULL )
#else
		if( getwd(cwd) == NULL )
#endif COHERENT
			goto wderr;
		continue;
	}
	else if ( !strcmp("pwd",token) ){
		printf("%s\n",cwd);
		continue;
	}
#endif CD

	/* Store the command name in the args variable. */
#ifdef NOPATHPROC
#if COHERENT
	if( access( BIN_PATH, AEXEC ) ){
#else
	if( access( BIN_PATH, F_OK | X_OK ) ){
#endif COHERENT
		fprintf(stderr,"%s : command not found.\n",token);
		continue;
	}
	strcpy(aargs[0],BIN_PATH);
	strcat(aargs[0],token);
	args[0] = aargs[0];
#else
	if( ( args[0] = path( BIN_PATH, token, AEXEC )) == NULL ){
		fprintf(stderr,"%s : command not found.\n",token);
		continue;
	}
#endif NOPATHPROC

	numargs++;

	/* Begin argument-grabbing loop. */
   while(1){
	p += strlen(token)+1;

	if( *p == '\0' )
		goto dofork;

	/* Skip whitespaces. */
	while( *p == ' ' || *p == '\t' )
		p++;

	if( *p == '\n' )
		goto dofork;

	/* Else it's an argument. */
	if( sscanf(p,"%s",token) != 1 )
		goto err;

	strcpy(aargs[numargs], token);
	args[numargs] = aargs[numargs];
	numargs++;

   }  /* While */
	
dofork:	args[numargs] = 0;
	pid = fork();
	if ( pid != 0 ){
		pid = 0;
		wait( &pid );
	}
	else {
		execv( args[0], args );
		perror(args[0]);
		exit( -1 );
	}
} /* While */
} /* main */
